#!/usr/bin/env python

"""
enumerate

The optional argument added to enumerate by the enumerate package will be
implemented in the Base/LaTeX/Lists.py package.  It simply ignores the argument
at the moment.

"""
